package main

import (
	"errors"
	"fmt"
	tgbot "github.com/go-telegram-bot-api/telegram-bot-api"
	"github.com/umputun/go-flags"
	"io/ioutil"
	"log"
	"os"
	"strings"
	"valera/commands"
)

const (
	version = "v0.1.1"
	userStateNone = userState("None")
	userStateGo = userState("Go")
)

var (
	state = userStateNone
)

type userState string

type Opts struct {
	Token string `short:"t" long:"token" description:"Telegram api token"`
	Name  string `short:"n" long:"name" description:"Telegram bot name" default:"@body_weight_loss_bot"`
}

var opts Opts

func readToken() (string, error) {
	b, err := ioutil.ReadFile("token.txt")
	if err != nil {
		return "", err
	}
	str := string(b)
	return str, nil
}

func main() {
	run()
}

func run() {
	p := flags.NewParser(&opts, flags.PrintErrors|flags.PassDoubleDash|flags.HelpFlag)
	p.SubcommandsOptional = true
	if _, err := p.Parse(); err != nil {
		if err.(*flags.Error).Type != flags.ErrHelp {
			log.Println(errors.New("[ERROR] cli error: " + err.Error()))
		}
		os.Exit(2)
	}

	if opts.Token == "" {
		token, err := readToken()
		if err != nil {
			panic(err)
		}
		opts.Token = strings.ReplaceAll(token, "\n", "")
	}
	fmt.Println(opts.Token)

	bot, err := tgbot.NewBotAPI(opts.Token)
	if err != nil {
		panic(err)
	}

	u := tgbot.NewUpdate(0)
	u.Timeout = 60

	updates, err := bot.GetUpdatesChan(u)
	if err != nil {
		panic(err)
	}

	log.Println("Run", opts.Name)

	for update := range updates {

		if update.Message == nil {
			continue
		}

		text := update.Message.Text
		chatID := update.Message.Chat.ID
		// username := update.Message.From.UserName

		if state == userStateGo {
			_, _ = bot.Send(tgbot.NewMessage(chatID, text))
			state = userStateNone
			continue
		}

		command := commands.Command(strings.Replace(text, opts.Name, "", 1))
		switch command {
		case commands.Start:
			_, _ = bot.Send(tgbot.NewMessage(chatID, fmt.Sprintf("Здорова, я Валера (%s), твой тренер (%d).", version, chatID)))
		case commands.Help:
			_, _ = bot.Send(tgbot.NewMessage(chatID, "Вот что я умею:\n\n1) Предлагать размяться"))
		case commands.Ping:
			_, _ = bot.Send(tgbot.NewMessage(chatID, "pong"))
		case commands.Go:
			msg := tgbot.NewMessage(chatID, "Давай немного разомнемся, отжимания")
			msg.ReplyMarkup = tgbot.NewReplyKeyboard(
				tgbot.NewKeyboardButtonRow(
					tgbot.NewKeyboardButton("1"),
					tgbot.NewKeyboardButton("2"),
					tgbot.NewKeyboardButton("3"),
				),
			)
			if _, err = bot.Send(msg); err == nil {
				state = userStateGo
			}
		}
	}
}
